"""Target annotator

Created by:
EOLE (http://eole.orion.education.fr)
Copyright (C) 2005-2018

Forked by:
Cadoles (http://www.cadoles.com)
Copyright (C) 2019-2021

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
from ..i18n import _
from ..error import DictConsistencyError


class TargetAnnotator:
    """Target annotator
    """
    def convert_target(self,
                       objects,
                       ) -> None:
        """ valid and convert param
        """
        targets = []
        for obj in objects:
            if not hasattr(obj, 'target'):
                msg = _('target is mandatory')
                raise DictConsistencyError(msg, 9, obj.xmlfiles)
            remove_targets = []
            for index, target in enumerate(obj.target):
                # test if it's redefined calculation
                if self.target_is_uniq and target.name in targets:
                    msg = _(f'A fill already exists for the target of "{target.name}" created')
                    raise DictConsistencyError(msg, 24, obj.xmlfiles)
                targets.append(target.name)

                # let's replace the target by the path
                try:
                    if target.type == 'variable':
                        path, suffix = self.objectspace.paths.get_variable_path(target.name,
                                                                                obj.namespace,
                                                                                )
                        target.name = self.objectspace.paths.get_variable(path)
                        if suffix:
                            msg = _(f'target to {target.name.path} with suffix is not allowed')
                            raise DictConsistencyError(msg, 35, obj.xmlfiles)
                    elif self.only_variable:
                        msg = _(f'target to "{target.name}" with param type "{target.type}" '
                                f'is not allowed')
                        raise DictConsistencyError(msg, 8, obj.xmlfiles)
                    if target.type == 'family':
                        if obj.name not in ['disabled_if_in',
                                            'disabled_if_not_in',
                                            'hidden_if_in',
                                            'hidden_if_not_in',
                                            ]:
                            msg = _(f'target "{target.type}" not allow with "{obj.name}"')
                            raise DictConsistencyError(msg, 51, target.xmlfiles)
                        target.name = self.objectspace.paths.get_family(target.name,
                                                                        obj.namespace,
                                                                        )
                    elif target.type.endswith('list') and \
                            obj.name not in ['disabled_if_in', 'disabled_if_not_in']:
                        msg = _(f'target "{target.type}" not allow with "{obj.name}"')
                        raise DictConsistencyError(msg, 10, target.xmlfiles)
                except DictConsistencyError as err:
                    if err.errno != 42:
                        raise err
                    # for optional variable
                    if not target.optional:
                        msg = f'cannot found target "{target.type}" "{target.name}"'
                        raise DictConsistencyError(_(msg), 12, target.xmlfiles) from err
                    remove_targets.append(index)
            remove_targets.sort(reverse=True)
            for index in remove_targets:
                obj.target.pop(index)
